#ifndef Men_H
#define Men_H

#include <Knopf.h>
#include <Tabelle.h>
#include <Diagramm.h>
#include <AuswahlBox.h>
#include "Map.h"

using namespace Framework;

class MenVerloren : public virtual ReferenceCounter
{
private:
    Rahmen *ram;
    TextFeld *verloren;
    Knopf *neu;
    Knopf *beenden;

public:
    // Kontruktor
    MenVerloren( UIInit &uiFactory );
    // Destruktor
    ~MenVerloren();
    // nicht constant
    int doPublicMausEreignis( MausEreignis &me );
    bool tick( double tickVal );
    void render( Bild &zRObj );
};

class MenSpiel : public virtual ReferenceCounter
{
private:
    TextFeld *scoreT;
    Knopf *beenden;
    Map *map;
    MenVerloren *verloren;
    double timePuffer;
    bool beendet;

public:
    // Konstruktor
    MenSpiel( UIInit &uiFactory, KSGClient::MinigameServerClient *klient );
    // Destruktor
    ~MenSpiel();
    // nicht constant
    void reset();
    void doPublicMausEreignis( MausEreignis &me );
    void doTastaturEreignis( TastaturEreignis &te );
    bool tick( double tickVal );
    void render( Bild &zRObj );
    // constant
    bool istBeendet() const;
};

class MenWiederhohlung : public virtual ReferenceCounter
{
private:
    TextFeld *scoreT;
    Knopf *beenden;
    Map *map;
    Datei *datei;
    double nowTime;
    double nextTime;
    bool beendet;
    double timePuffer;

public:
    // Konstruktor
    MenWiederhohlung( UIInit &uiFactory, Datei *datei, Text *zOptionen );
    // Destruktor
    ~MenWiederhohlung();
    // nicht constant
    void doPublicMausEreignis( MausEreignis &me );
    void doTastaturEreignis( TastaturEreignis &te );
    bool tick( double tickVal );
    void render( Bild &zRObj );
    // constant
    bool istBeendet() const;
};

class MenStatistik : public virtual ReferenceCounter
{
private:
    TextFeld *ansichtT;
    AuswahlBox *ansicht;
    KontrollKnopf *optionen;
    ObjTabelle *gesammtT;
    ObjTabelle *optionenT;
    ObjTabelle *worldBestT;
    LDiag *gesammtD;
    LDiag *optionenD;
    Knopf *removen;
    Knopf *zurck;
    UIInit uiFactory;
    TextRenderer *tr;
    KSGClient::MinigameServerClient *klient;
    MenWiederhohlung *wiederH;
    bool asyncFinished;
    bool beendet;

public:
    // Konstruktor
    MenStatistik( UIInit &uiFactory, KSGClient::MinigameServerClient *klient );
    // Destruktor
    ~MenStatistik();
    // nicht constant
    void reset();
    void doPublicMausEreignis( MausEreignis &me );
    bool tick( double tickVal );
    void render( Bild &zRObj );
    // constant
    bool istBeendet() const;
    bool istWiederhohlung() const;
};

class MenOptionen : public virtual ReferenceCounter
{
private:
    TextFeld *breiteT;
    TextFeld *breite;
    TextFeld *hheT;
    TextFeld *hhe;
    TextFeld *neuTempoT;
    TextFeld *neuTempo;
    TextFeld *tempoT;
    TextFeld *tempo;
    KontrollKnopf *fortsetzen;
    Knopf *abbrechen;
    Knopf *ok;
    bool beendet;

public:
    // Konstruktor
    MenOptionen( UIInit &uiFactory );
    // Destruktor
    ~MenOptionen();
    // nicht constant
    void reset();
    void doPublicMausEreignis( MausEreignis &me );
    void doTastaturEreignis( TastaturEreignis &te );
    bool tick( double tickVal );
    void render( Bild &zRObj );
    // constant
    bool istBeendet() const;
};

class Men : public virtual ReferenceCounter
{
private:
    Knopf *spielen;
    MenSpiel *spielenF;
    Knopf *optionen;
    MenOptionen *optionenF;
    Knopf *statistik;
    MenStatistik *statistikF;
    Knopf *beenden;
    Bild *hintergrund;
    Rahmen *ram;
    bool beendet;
    int status;

public:
    // Konstruktor
    Men( UIInit &uiFactory, KSGClient::MinigameServerClient *klient );
    // Destruktor
    ~Men();
    // nicht constant#
    void reset();
    void doPublicMausEreignis( MausEreignis &me );
    void doTastaturEreignis( TastaturEreignis &te );
    bool tick( double z );
    void render( Bild &zRObj );
    // constant
    bool istBeendet() const;
};

#endif